@@ -19,7 +19,7 @@ downloadCivEDICCS <- function(years=c(1999, 2009)) {
     stop(paste0("The argument ", sQuote("years"), " must have values of only ", sQuote("1999"), " or ", sQuote("2009"), "."))
   }
   
-  linkURL <- "https://www.iea.nl/data"
+  linkURL <- "https://www.iea.nl/data-tools/repository"
   
   txt <- c()
   txt <- c(txt, paste0("Please manually download and extract the SPSS (*.sav) formatted CivED 1999 or ICCS 2009 study data files from the IEA Data Repository to a folder on your local system or network. ",
@@ -27,14 +27,11 @@ downloadCivEDICCS <- function(years=c(1999, 2009)) {
                        "See help page (?readCivEDICCS) for more details."))
   txt <- c(txt, "\n")
   
-  txt <- c(txt, paste0("\t", "1) Launch the IEA Data Repository web URL (", linkURL ,") in your web browser.",
-                       " An IEA Account is required to download the data, so either create one following the IEA instructions, or login with an existing account."))
+  txt <- c(txt, paste0("\t", "1) Launch the IEA Data Repository web URL (", linkURL ,") in your web browser."))
   
-  txt <- c(txt, paste0("\t", "2) Click on the ", dQuote("here"), " link under ", dQuote("The IEA Data Repository"), " section to proceed, ", 
-                       "then click the ", dQuote("1. IEA Data Repository"), " link to view all available studies for download. ",
-                       "Find your study (CivED or ICCS) and click the appropriate studies ", dQuote("year"), " link you wish to download."))
+  txt <- c(txt, paste0("\t", "2) Find your study (CivED or ICCS) and click the appropriate studies ", dQuote("year"), " link you wish to download."))
   
-  txt <- c(txt, paste0("\t", "3) Next, select the link ", dQuote("Download SPSS Data & Documentation"), ", because the SPSS (*.sav) files are compatible with the ", dQuote("readCivEDICCS()"), " function in EdSurvey. ",
+  txt <- c(txt, paste0("\t", "3) Next, click on the link ", dQuote("SPSS Data & Documentation"), " to download the SPSS (*.sav) files, which are compatible with the ", dQuote("readCivEDICCS()"), " function in EdSurvey. ",
                        "Follow your web browser's prompts to download the resulting *.zip file to a folder location you can find later."))
   
   txt <- c(txt, paste0("\t", "4) Locate your downloaded zip file (*.zip) container and use an extraction program to extract the folder's file contents. ",
