@@ -1,4 +1,6 @@
+\dontrun{
 # read in the example data (generated, not real student data)
 sdf <- readNAEP(system.file("extdata/data", "M36NT2PM.dat", package="NAEPprimer"))
 
 getPlausibleValue(var="composite", data=sdf)
+}
