@@ -1,3 +1,4 @@
+\dontrun{
 # read in NAEP primer data
 sdf <- readNAEP(system.file("extdata/data", "M36NT2PM.dat", package = "NAEPprimer"))
 lsdf <- getData(data=sdf, varnames=c("dsex", "b017451"), addAttributes = TRUE)
@@ -14,4 +15,4 @@ df2 <- data.frame(dsex = c("Male","Female"), b017451 = c(1,2))
 merged_lsdf2 <- merge(lsdf,df2, by = "dsex")
 names(merged_lsdf2) # "dsex"      "b017451.x" "b017451.y"
 head(merged_lsdf2) # shows merge results
-
+}
