@@ -6,6 +6,8 @@ fin <- readTIMSS("~/TIMSS/2015", countries="fin", gradeLvl=4)
 mix1 <- mixed.sdf(mmat ~ itsex + (1|idschool), data = fin,
                   weightVar=c("totwgt","schwgt"), weightTransformation=FALSE)
 summary(mix1)
+# a trivial Wald test example
+waldTest(mix1, "itsex")
 # uses only one plausible value
 mix2 <- mixed.sdf(asmmat01 ~ itsex + (1|idschool), data = fin,
                   weightVar=c("totwgt","schwgt"), weightTransformation=FALSE)
