@@ -1,3 +1,4 @@
+\dontrun{
 # read in the example data (generated, not real student data)
 sdf <- readNAEP(system.file("extdata/data", "M36NT2PM.dat", package="NAEPprimer"))
 
@@ -7,3 +8,4 @@ sdf2 <- updatePlausibleValue("composite", "overall", sdf)
 showPlausibleValues(sdf2)
 lm1 <- lm.sdf(overall ~ b017451, data=sdf2)
 summary(lm1)
+}
