@@ -61,17 +61,27 @@ et2REF <- c("",
             "JK replicates: 62",
             "",
             "Summary Table:",
-            "      labels              b017451  N   WTD_N      PCT", 
-            " B locations Never or hardly ever 19 27.9876 24.81978",
-            " B locations Once every few weeks 17 24.0644 21.34063", 
-            " B locations    About once a week 10 13.8264 12.26144",
-            " B locations  2 or 3 times a week 16 24.4471 21.68001", 
-            " B locations            Every day 14 22.4378 19.89814",
-            " C locations Never or hardly ever 15 22.6297 25.45744", 
-            " C locations Once every few weeks 10 10.8126 12.16371",
-            " C locations    About once a week 11 16.6016 18.67608", 
-            " C locations  2 or 3 times a week 10 13.3960 15.06992",
-            " C locations            Every day 15 25.4524 28.63285"
+            "             labels              b017451  N WTD_N PCT", 
+            "  first E locations Never or hardly ever NA    NA  NA",
+            "  first E locations Once every few weeks NA    NA  NA",
+            "  first E locations    About once a week NA    NA  NA",
+            "  first E locations  2 or 3 times a week NA    NA  NA",
+            "  first E locations            Every day NA    NA  NA",
+            "        B locations Never or hardly ever 19    28  25",
+            "        B locations Once every few weeks 17    24  21", 
+            "        B locations    About once a week 10    14  12",
+            "        B locations  2 or 3 times a week 16    24  22", 
+            "        B locations            Every day 14    22  20",
+            "        C locations Never or hardly ever 15    23  25", 
+            "        C locations Once every few weeks 10    11  12",
+            "        C locations    About once a week 11    17  19", 
+            "        C locations  2 or 3 times a week 10    13  15",
+            "        C locations            Every day 15    25  29",
+            " second E locations Never or hardly ever NA    NA  NA",
+            " second E locations Once every few weeks NA    NA  NA",
+            " second E locations    About once a week NA    NA  NA",
+            " second E locations  2 or 3 times a week NA    NA  NA",
+            " second E locations            Every day NA    NA  NA"
 )
 
 printREF <- c("an edsurvey.data.frame.list with 4 elements",
