@@ -147,56 +147,52 @@ probit1REF <- c("",
                 "---",
                 "Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1")
 
-spv <-  c("There are 16 subject scale(s) or subscale(s) in this edsurvey.data.frame:", 
-          "'mibm' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'asmibm01', 'asmibm02', 'asmibm03', 'asmibm04', and 'asmibm05'", 
-          "",
-          "'sibm' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'assibm01', 'assibm02', 'assibm03', 'assibm04', and 'assibm05'", 
-          "",
-          "'mmat' subject scale or subscale with 5 plausible values (the default).", 
-          "  The plausible value variables are: 'asmmat01', 'asmmat02', 'asmmat03', 'asmmat04', and 'asmmat05'", 
-          "",
-          "'ssci' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'asssci01', 'asssci02', 'asssci03', 'asssci04', and 'asssci05'", 
-          "",
-          "'mdat' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'asmdat01', 'asmdat02', 'asmdat03', 'asmdat04', and 'asmdat05'", 
-          "",
-          "'mgeo' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'asmgeo01', 'asmgeo02', 'asmgeo03', 'asmgeo04', and 'asmgeo05'", 
-          "",
-          "'mnum' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'asmnum01', 'asmnum02', 'asmnum03', 'asmnum04', and 'asmnum05'", 
-          "",
-          "'sear' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'assear01', 'assear02', 'assear03', 'assear04', and 'assear05'", 
-          "",
-          "'slif' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'asslif01', 'asslif02', 'asslif03', 'asslif04', and 'asslif05'", 
-          "",
-          "'sphy' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'assphy01', 'assphy02', 'assphy03', 'assphy04', and 'assphy05'", 
-          "",
-          "'mkno' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'asmkno01', 'asmkno02', 'asmkno03', 'asmkno04', and 'asmkno05'", 
-          "",
-          "'mapp' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'asmapp01', 'asmapp02', 'asmapp03', 'asmapp04', and 'asmapp05'", 
-          "",
-          "'mrea' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'asmrea01', 'asmrea02', 'asmrea03', 'asmrea04', and 'asmrea05'", 
-          "",
-          "'skno' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'asskno01', 'asskno02', 'asskno03', 'asskno04', and 'asskno05'", 
-          "",
-          "'sapp' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'assapp01', 'assapp02', 'assapp03', 'assapp04', and 'assapp05'", 
-          "",
-          "'srea' subject scale or subscale with 5 plausible values.", 
-          "  The plausible value variables are: 'assrea01', 'assrea02', 'assrea03', 'assrea04', and 'assrea05'", 
+#for whatever reason, the sQuote behaves really odd.  change the ‘srea’ to 'srea'.  all.equal will show false, but expect_equal must coerce it somehow?
+spv <-  c("There are 14 subject scale(s) or subscale(s) in this edsurvey.data.frame:",                       
+          "'mmat' subject scale or subscale with 5 plausible values (the default).",                         
+          "  The plausible value variables are: 'asmmat01', 'asmmat02', 'asmmat03', 'asmmat04', and 'asmmat05'",
+          "",                                         
+          "'ssci' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'asssci01', 'asssci02', 'asssci03', 'asssci04', and 'asssci05'",
+          "",                                         
+          "'mdat' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'asmdat01', 'asmdat02', 'asmdat03', 'asmdat04', and 'asmdat05'",
+          "",                                         
+          "'mgeo' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'asmgeo01', 'asmgeo02', 'asmgeo03', 'asmgeo04', and 'asmgeo05'",
+          "",                                         
+          "'mnum' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'asmnum01', 'asmnum02', 'asmnum03', 'asmnum04', and 'asmnum05'",
+          "",                                         
+          "'sear' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'assear01', 'assear02', 'assear03', 'assear04', and 'assear05'",
+          "",                                         
+          "'slif' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'asslif01', 'asslif02', 'asslif03', 'asslif04', and 'asslif05'",
+          "",                                         
+          "'sphy' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'assphy01', 'assphy02', 'assphy03', 'assphy04', and 'assphy05'",
+          "",                                         
+          "'mkno' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'asmkno01', 'asmkno02', 'asmkno03', 'asmkno04', and 'asmkno05'",
+          "",                                         
+          "'mapp' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'asmapp01', 'asmapp02', 'asmapp03', 'asmapp04', and 'asmapp05'",
+          "",                                         
+          "'mrea' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'asmrea01', 'asmrea02', 'asmrea03', 'asmrea04', and 'asmrea05'",
+          "",                                         
+          "'skno' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'asskno01', 'asskno02', 'asskno03', 'asskno04', and 'asskno05'",
+          "",                                         
+          "'sapp' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'assapp01', 'assapp02', 'assapp03', 'assapp04', and 'assapp05'",
+          "",                                         
+          "'srea' subject scale or subscale with 5 plausible values.",
+          "  The plausible value variables are: 'assrea01', 'assrea02', 'assrea03', 'assrea04', and 'assrea05'",
           "")
 
+
 assignTest1 <- c("    asmrea01        asmrea02        asmrea03        asmrea04        asmrea05    ", 
                  " Min.   :  5.0   Min.   :  5.0   Min.   :  5.0   Min.   :  5.0   Min.   :  5.0  ",
                  " 1st Qu.:246.5   1st Qu.:246.8   1st Qu.:249.8   1st Qu.:248.5   1st Qu.:247.3  ",
