@@ -26,3 +26,14 @@ test_that('mixed.sdf', {
                       co <- capture.output(summary(m1)))
   expect_equal(co, m1SummaryREF)
 })
+
+context('mixed.sdf Wald test')
+test_that('mixed.sdf Wald test', {
+  cntl <- readPISA(paste0(edsurveyHome, "PISA/2012"), countries="USA", verbose=FALSE)
+  # model with one random effect
+  m1 <- mixed.sdf(math ~ st29q03 + sc14q02 +st04q01+escs+ (1|schoolid),
+                  data=cntl)
+  wt <- waldTest(m1, coef=2:4)
+  expect_is(wt, "edsurveyWaldTest")
+  expect_equal(unname(wt$result$chi2["chi2"]), 30.2098, tol=200*sqrt(.Machine$double.eps))
+})
